\\ ---------- Methods for the computation of Frobenius polynomials ----------

MUMSOL(L,ord)={
    local(f=[1,0,0,0]);
    L=matrix(4,poldegree(L,z)+1,X,Y,diffop(polcoef(L,Y-1,z),[Theta],[1],X-1));
    for(X=1,4,
        for(k=1,ord,
            f[X]-=sum(i=0,min(#L-1,k),sum(j=1,X,subst(L[j,i+1],Theta,k-i)/(j-1)!*polcoef(f[X-j+1],k-i,z)))/subst(L[1,1],Theta,k)*z^k
        );
        f[X]+=O(z^(ord+1))
    );
    f
};

BMATRIX(f)={
	local(df=z*deriv(f,z),ddf=z*deriv(df,z),dddf=z*deriv(ddf,z));
	[f[1],df[1],ddf[1],dddf[1];f[2],f[1]+df[2],2*df[1]+ddf[2],3*ddf[1]+dddf[2];f[3],f[2]+df[3],f[1]+2*df[2]+ddf[3],3*df[1]+3*ddf[2]+dddf[3];f[4],f[3]+df[4],f[2]+2*df[3]+ddf[4],f[1]+3*df[2]+3*ddf[3]+dddf[4]]
};

FROB(L,c,ord,plist)={
    local(TRUNC=(M,n)->matrix(#M,#M,X,Y,M[X,Y]+O(z^(n+1))));
    local(f,b,binv,binvp,p,frob);
    f=MUMSOL(L,ord(plist[#plist]));
    b=BMATRIX(f);
    binv=matsolve(TRUNC(b,ord(1)),matid(4));
    print("Computed Frobenius matrix for the following primes:");
    for(k=1,#plist,
        p=plist[k];
        binvp=TRUNC(subst(binv,z,z^p),ord(p));
        frob=binvp*[1,0,0,0;0,p,0,0;0,0,p^2,0;c*ZETA3(p)*p^3,0,0,p^3]*TRUNC(b,ord(p));
        frob=truncate(lift(frob+O(p^4)*sum(i=0,ord(p),z^i)*matrix(4,4,X,Y,1)));
        plist[k]=[p,frob];
	print1(p," ");
    );
    print();
    plist
};

FROBPOL(frob,xlist)={
    local(p,tr1,tr2);
    p=frob[1];
    tr1=-trace(frob[2]);
    tr2=(tr1^2-trace(frob[2]^2))/(2*p);
    xlist=lift(apply(x->centerlift(subst([x,tr1+O(p^3),tr2+O(p^3)],z,teichmuller(x+O(p^3)))),xlist));
    xlist=apply(v->[v[1],if(v[2]<=4*p^(3/2),v[2],v[2]-p^3),if(v[3]<=6*p^2,v[3],v[3]-p^3)],xlist);
    apply(x->[x[1],1+x[2]*T+p*x[3]*T^2+p^3*x[2]*T^3+p^6*T^4],xlist)
};

\\ Numerical computation of p-adic zeta(3)
ZETA3(p) = {
  local(e=p^5,n=20);
  -(gamma(3*e+O(p^n))-3*gamma(2*e+O(p^n))+3*gamma(e+O(p^n))-1-(gamma(e+O(p^n))-1)^3)/(2*e^3) + O(p^10)
};



\\ ---------- Computations ----------

default(parisize,"1G")

L = Theta^4-3^6*z*(Theta+1/3)^2*(Theta+2/3)^2;
c = 16;
ord = x->ceil(2/3*x);
zStar = -1/(2^3*3^6);

primeList = primes(primepi(1000));
primeList = primeList[5..#primeList];

frob = FROB(L,c,ord,primeList);

points = vector(#frob,i,[frob[i][1],select(x->Mod(subst(z*polcoef(L,4,Theta),z,x),frob[i][1])!=0,vector(frob[i][1],j,j-1))]);
print("\nComputed characteristic polynomials for the following primes:");
pols = vector(#frob,i,print1(frob[i][1]," ");[frob[i][1],FROBPOL(frob[i],points[i][2])]);
print();

factorizations = apply(v->[v[1],select(w->poldegree(factor(w[2])[1,1])<4,v[2])],pols);
print("\nNumbers of factorizations over Z:");
print(apply(v->[v[1],#v[2]],factorizations));

print("\nFactorization for polynomials of reductions of zStar:");
apply(v->print([v[1],factor(select(w->Mod(w[1]-zStar,v[1])==0,v[2])[1][2])]),pols);
