\\ ---------- Methods for the point counting ----------

S(z,s,t,K) = if(K==0,ellinit([0,1+16*z*(t/s)^2,0,16*z*(t/s)^2,0]),ellinit(Mod([0,1+16*z*(t/s)^2,0,16*z*(t/s)^2,0],K.pol),K));

countS(z,s,t) = {
	    local(p=s.p,k=poldegree(s.mod));
	    if(s==0,return(if(#polrootsmod(x^2-z,z), 4*p^k,4*(p^k+1))));
	    if(t==0,return(4*p^k));
	    if(s^2-16*z*t^2==0,return(if(#polrootsmod(x^2+1,z), 2*p^k, 2*(p^k+1))));
	    return(S(z,s,t).no);
};

countX(z,p,k) = {
	 local(modP,gen1,gen2,fieldElements,n);
	 p = idealfactor(K,p)[1,1];
	 modP = nfmodprinit(K, p);
	 z = nfmodpr(K,z,modP);
	 gen1 = ffgen(z);
	 gen2 = ffgen(idealnorm(K,p)^k);
	 [p,k] = [gen2.p,poldegree(gen2.mod)];
	 z = ffmap(ffembed(gen1, gen2), z);
	 fieldElements = List();
	 forvec(X=vector(k,i,[0,p-1]),listput(fieldElements,sum(i=0,k-1,X[i+1]*gen2^i)));
	 n = 0;
	 n += countS(1+0*gen2,0*gen2,1+0*gen2)*countS(z,1+0*gen2,0*gen2);
	 for(t=1,#fieldElements,n += countS(1+0*gen2,1+0*gen2,fieldElements[t])*countS(z,fieldElements[t],1+0*gen2));
	 n + if(#polrootsmod(x^2-z,z), 32*p^k, 16*p^k)
};

fixedPointsPhiFrob(p) = {
	local(modP,xMod,gen1,gen2,np,ts,xs,ys,f,n1,n2);
	p = idealfactor(K,p)[1,1];
	modP = nfmodprinit(K,p);
	xMod = nfmodpr(K,X,modP);
	np = idealnorm(K,p);
	gen1 = ffgen(xMod);
	gen2 = ffgen(np^4);
	[p,k] = [gen2.p,poldegree(gen2.mod)];
	xMod = ffmap(ffembed(gen1, gen2), xMod);
	ts = polrootsmod(x^(np+1)+1/16,gen2)~;
	sum(t=1,#ts,
		xs = polrootsmod((x/(4*ts[t])^2)^np-x,gen2)~;
		ys = polrootsmod((x/(4*ts[t])^3)^np-x,gen2)~;
		n1 = 1;
		for(x=1,#xs,for(y=1,#ys,if(substvec(yy^2-xx*(xx+1)*(xx+16*ts[t]^2),[xx,yy],[xs[x],ys[y]])==0,n1++)));
		xs = polrootsmod((x/(4*(xMod/16)/ts[t])^2)^np-x,gen2)~;
		ys = polrootsmod((x/(4*(xMod/16)/ts[t])^3)^np-x,gen2)~;
		n2 = 1;
		for(x=1,#xs,for(y=1,#ys,if(substvec(yy^2-xx*(xx+1)*(xx+16*(-1/2^8)/ts[t]^2),[xx,yy],[xs[x],ys[y]])==0,n2++)));
		n1*n2
	)
};



\\ ---------- Computations ----------

default(parisize,"1G")

K = nfinit(X^2+1);

primeList = [2+X,3,2+3*X];
zStar = -1/2^8;
zStarSqrt = nfroots(K,x^2-zStar)[1];

\\ The curves E1, E2, E3, E4 are all equal to E. 
E1 = S(zStar,1,-4,K);
E2 = S(zStar,1,4,K);
E3 = S(1,1,-4*zStarSqrt,K);
E4 = S(1,1,4*zStarSqrt,K);
E = ellinit([0,0,0,-1,0],K);

computeData(p) = {
	my(q,pIdeal,lFrob,lFrob2,lPhiFrob,trFrob,trFrob2,pol,trDiff);
	pIdeal = idealfactor(K,p)[1,1];
	q = idealnorm(K,p);
	lFrob = countX(zStar,p,1);
	lFrob2 = countX(zStar,p,2);
	trFrob = 1 + 37*q - 4*q*ellap(E,pIdeal) + 37*q^2 + q^3 - lFrob;
	trFrob2 = 1 + 37*q^2 - 4*q^2*(ellap(E,pIdeal)^2-2*q) + 37*q^4 + q^6 - lFrob2;
	pol = substvec(1+A*T+q*B*T^2+A*q^3*T^3+q^6*T^4,[A,B],[-trFrob,(trFrob^2-trFrob2)/(2*q)]);
	trDiff = factor(pol,I)[1,1];
	trDiff = trDiff/subst(trDiff,T,0);
	pol = [trDiff,pol/trDiff];
	trDiff = polcoef(pol[1]-pol[2],1,T);
	lPhiFrob=fixedPointsPhiFrob(p);
	if(lPhiFrob!=1+3*q-I*trDiff+3*q^2+q^3,pol=[pol[2],pol[1]]);
	[subst(p,X,I),lFrob,lFrob2,lPhiFrob,pol[1]*pol[2],pol[1],pol[2],ellap(E,pIdeal)]
};

print("Computing [p, L(F_p), L(F_p^2), L(phi*F_p), det(1-T*F_p|V), det(1-T*F_p|V-), det(1-T*F_p|V+), tr(F_p|H^1(E))]:");
apply(p->print(computeData(p)),primeList);
